/** @file AutoRefresh.c
* AutoRefresh : MSX-E302x : Auto Refresh acquisition
* 
*
* @author C. Guinot
* @date 12.12.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : -   
*  Author        : - 
*  Description   : -
*
* @endverbatim     
*
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MSXE302x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"

#define NBR_OF_CHANNELS 16

/* Console management functions */
int    i_X             = 0;
int    i_Y             = 0;
INT    i_ThreadStartY  = 0;
INT    i_ThreadCurentY = 0;

HANDLE h_ConsoleHandle = NULL;
HANDLE h_ConsoleMutex  = NULL;


/*******************************************************************************************************/
/*                                       CONSOLE Functions                                             */               
/*******************************************************************************************************/

/*gotoxy*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void gotoxy (int X, int Y)                             |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Set the cursor to the selected postion                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : int X : Coordinate X                                   |*/
/*|                     int Y : Coordinate Y                                   |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void gotoxy(int X, int Y)
{
	/* Variable declaration */
	COORD s_CoordScreen = { 0, 0 };
        
	/* Test if consol handle open */
	if (h_ConsoleHandle != NULL)
	{
		/* Save the new coordinate */
		s_CoordScreen.X = X;
		s_CoordScreen.Y = Y;

		/* Put the cursor at (X, Y) */
		SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);
	}
}

/*wherexy*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void wherexy (int *X, int *Y)                          |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Get the cursor postion                                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : int * X : Coordinate X                                 |*/
/*|                     int * Y : Coordinate Y                                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void wherexy (int *X, int *Y)
{
	/* Variable declaration */
	CONSOLE_SCREEN_BUFFER_INFO  csbi;

	/* Test if consol handle open */
	if (h_ConsoleHandle != NULL)
	{
		/* get the number of character cells in the current buffer */
		GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

		/* Get the current position  */
		*X = csbi.dwCursorPosition.X;
		*Y = csbi.dwCursorPosition.Y;
	}
}

/*clrscr*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void clrscr (void)                                     |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Clear the screen and set the cursor to postion 0, 0    |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void clrscr(void)
{
	/* Variable declaration */
	DWORD                      dw_ConSize;
	DWORD                      dw_CharsWritten; 
	COORD                       s_CoordScreen = { 0, 0 };
	CONSOLE_SCREEN_BUFFER_INFO  csbi;

	/* Test if consol handle open */
	if (h_ConsoleHandle != NULL)
	{
		/* get the number of character cells in the current buffer */
		GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

		/* Get the number of blank to set */
		dw_ConSize = csbi.dwSize.X * csbi.dwSize.Y; 

		/* Fill the entire screen with blanks */
		FillConsoleOutputCharacter(h_ConsoleHandle, 
									(TCHAR) ' ',
									dw_ConSize,
									s_CoordScreen,
									&dw_CharsWritten ); 

		/* Put the cursor at (0, 0) */
		SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);
		wherexy (&i_X, &i_Y);
	}
}

/*SetNewPosition*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void SetNewPosition (int i_YToInsert)                  |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Test and set the number of line to insert              |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : i_YToInsert : Number of line to insert                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void SetNewPosition(int i_YToInsert)
{
	/* Variable declaration */
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	/* Get the current cursor position and the max X, Y */
	GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

	/* Test the number of line to insert */
	if ((csbi.dwCursorPosition.Y + i_YToInsert) >= csbi.dwSize.Y)
	{
		/* Clear the screan */
		clrscr();
	}
	else
	{
		/* Save te new Y position */
		i_Y = csbi.dwCursorPosition.Y + i_YToInsert;
	}
}

/*TestPosition*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void TestPosition (int i_YToInsert)                    |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Test the number of line to insert                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : i_YToInsert : Number of line to insert                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void TestPosition(int i_YToInsert)
{
	/* Variable declaration */
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	/* Get the current cursor position and the max X, Y */
	GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

	/* Test the number of line to insert */
	if ((csbi.dwCursorPosition.Y + i_YToInsert) >= csbi.dwSize.Y)
	{
		/* Clear the screan */
		clrscr();
	}
}


/*******************************************************************************************************/
/*                                       Get auto refresh parameter function                           */               
/*******************************************************************************************************/
void GetAutoRefreshParameters  (DWORD *pdw_ChannelType,
								DWORD *pdw_ChannelMask, 
								DWORD *pdw_ChannelTypeMask,
			                    DWORD *pdw_GainArray, 
			                    DWORD *pdw_PolarityArray, 
			                    DWORD *pdw_AverageMode, 
			                    DWORD *pdw_AverageValue, 
			                    DWORD *pdw_ConversionTime,
			                    DWORD *pdw_ConversionTimeUnit,
			                    DWORD *pdw_TriggerMask, 
			                    DWORD *pdw_TriggerMode, 
			                    DWORD *pdw_HardwareTriggerEdge, 
			                    DWORD *pdw_HardwareTriggerCount, 
			                    DWORD *pdw_ByTriggerNbrOfSeqToAcquire,
			                    DWORD *pdw_DataFormat,
								DWORD *pdw_AcquisitionMode)
{
	DWORD dw_Choice = 0;
    DWORD dw_Cpt = 0;
    DWORD dw_Temp = 0;
	DWORD dw_Nbr = 0;

	do
	{
		printf("\n0. Acquisition via the SOAP function.\n");
		printf("1. Acquisition via the data server.\n");
		printf("   It si recommended to set the bloking mode for the data server\n");
		printf("   via the web page.\n");
		printf("Select acquisition mode do You want to use: ");
		scanf("%d", pdw_AcquisitionMode);
		}
		while ((*pdw_AcquisitionMode != 0) &&(*pdw_AcquisitionMode != 1));


	/* CHANNEL MASK SELECTION */
    do
	{
		printf("\nEnter the channel mask (in Hex): ");
		scanf("%X", pdw_ChannelMask);
	}
	while ((*pdw_ChannelMask == 0) || (*pdw_ChannelMask > (pow(2,NBR_OF_CHANNELS) - 1)));


	/* CHANNEL INITIALISATION SELECTION */
    for (dw_Cpt = 0; dw_Cpt < NBR_OF_CHANNELS; dw_Cpt ++)
    {
        if ((*pdw_ChannelMask >> dw_Cpt) & 1)
        {
            printf("\n Channel %lu configuration :",dw_Cpt);
            if (pdw_ChannelType[dw_Cpt] == 0)
			{
				do
				{
					printf("\nEnter the gain(1 or 2):");
					scanf("%lu",&pdw_GainArray[dw_Cpt]);
				}
				while((pdw_GainArray[dw_Cpt] != 1) && (pdw_GainArray[dw_Cpt] != 2));
			}
			else
			{
				printf("\nChannel gain = 2");
				pdw_GainArray[dw_Cpt] = 2;
			}
            do
            {
                printf("\nEnter the polarity(0:Unipolar or 1:Bipolar):");
                scanf("%lu",&pdw_PolarityArray[dw_Cpt]);
            }
            while(pdw_PolarityArray[dw_Cpt] > 1);

			*pdw_ChannelTypeMask = *pdw_ChannelTypeMask | (pdw_ChannelType[dw_Cpt] << dw_Nbr++);

        }
    }


	/* AVERAGE MODE SELECTION */
    printf("\nDo You want to activate the average mode (0 or 1)? ");
	scanf("%d",&dw_Choice);

	if (dw_Choice == 1)
	{
		do
		{
			printf("\n1. Average mode per sequence\n");
			printf("2. Average mode per channel\n");
			printf("Select the Average mode do You want to use: ");
			scanf("%d", pdw_AverageMode);
		}
		while ((*pdw_AverageMode != 1) &&(*pdw_AverageMode != 2));

		do
		{
			printf("\nEnter the average value (2 to 255): ");
			scanf("%d", pdw_AverageValue);
		}
		while ((*pdw_AverageValue < 2) || (*pdw_AverageValue > 255));
	}
	else
	{
		*pdw_AverageValue = 0;
		*pdw_AverageMode = 0;
	}

    /* CONVERSION TIME SELECTION */
    do
    {
        printf("\nEnter the conversion time unit to use (0: us, 1: ms,2: s):");
        scanf ("%lu",pdw_ConversionTimeUnit);
    }
    while (*pdw_ConversionTimeUnit > 2);


    do
    {
        printf("\nEnter the conversion time to use (by us time unit : min value = 10us):");
        scanf ("%lu",pdw_ConversionTime);
    }
    while (((*pdw_ConversionTimeUnit == 0) && (*pdw_ConversionTime < 10)) ||
           (*pdw_ConversionTime > (pow(2,16)-1)));
	
	/* TRIGGER SELECTION */
	*pdw_TriggerMask = 0;
	*pdw_TriggerMode = 0;
	*pdw_HardwareTriggerEdge = 0;
	*pdw_HardwareTriggerCount = 0;
	*pdw_ByTriggerNbrOfSeqToAcquire = 0;

    printf("\nDo You want to use the hardware trigger (0 or 1)? ");
	scanf("%d", &dw_Choice);

	if (dw_Choice == 1)
	{
        *pdw_TriggerMask = *pdw_TriggerMask | 1;

		do 
		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			scanf ("%d", &dw_Choice);
		}
		while ((dw_Choice < 1) || (dw_Choice > 3));

		*pdw_HardwareTriggerEdge = dw_Choice;

		do
		{
			printf("\nEnter the number of edges before the trigger action occure (1 to 65535): ");
			scanf("%d",pdw_HardwareTriggerCount);
		}
		while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));


	}

	if (*pdw_TriggerMask == 0)
	{
	    printf("\nDo You want to use the Synchro trigger (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerMask = *pdw_TriggerMask | 2;

		}
	}


    if (*pdw_TriggerMask != 0)
    {
		do
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do You want to use: ");
			scanf("%d", pdw_TriggerMode);
		}
		while ((*pdw_TriggerMode != 1) && (*pdw_TriggerMode != 2));


    	if (*pdw_TriggerMode == 2)
        {
            do
	    	{
		    	printf("\nEnter the number of sequence to acquire by each trigger: ");
			    scanf("%d", pdw_ByTriggerNbrOfSeqToAcquire);
		    }
		    while (*pdw_ByTriggerNbrOfSeqToAcquire == 0);
        }


    }

	
    /* DATA FORMAT SELECTION */
    *pdw_DataFormat = 0;
	do 
	{
	    printf("\nDo You want to receive a time stamp (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = dw_Choice;

	do 
	{
	    printf("\nDo You want to receive the value in digital form (tape 0) or in analog form (type 1) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 1);
	

}



typedef struct
      {
      DWORD   dw_DataFormat;
      DWORD   dw_NumberOfChannelsToAcquire;
      BYTE    b_Exit;
      SOCKET  s_ClientSocket;
	  DWORD   dw_ChannelTypeMask;
	  DWORD   dw_ChannelMask;
	  struct  soap *ps_SoapEnv;
	  CHAR	  *pc_SOAPIPAddress;

      }str_ThreadInfo;



#pragma pack(1)
typedef union
      {
      struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtTimeStampBinaryData;

	 struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtTimeStampRealData;

	 struct
	    {
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtoutTimeStampBinaryData;


	 struct
	    {
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtoutTimeStampRealData;
	 }str_AutoRefreshData;
#pragma pack()



DWORD WINAPI dw_DataClientThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo * ps_ThreadInfo;
		INT i_ReturnValue             = 0;
    	DWORD dw_Cpt                  = 0;
    	DWORD dw_RequestValue         = 0;
    	DWORD dw_NbrOfReceiveValues   = 0;
    	DWORD dw_NbrOfValuesToReceive = 0;
		str_AutoRefreshData s_AutoRefreshData;
		BYTE *pb_ReadBuffer           = (BYTE *) &s_AutoRefreshData;
		CHAR *pc_Unit[2] = {"V","mA"};

        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {
		    /* Test if the data format time stamp is activated */
			if (ps_ThreadInfo->dw_DataFormat & 1)
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 3) * sizeof (DWORD);
			}
			else
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 1) * sizeof (DWORD);
			}
				
			dw_RequestValue = dw_NbrOfValuesToReceive;
			dw_NbrOfReceiveValues = 0;

			do
			{
				i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &pb_ReadBuffer[dw_NbrOfReceiveValues] , dw_RequestValue, 0); 

				if (i_ReturnValue != SOCKET_ERROR)
				{
					dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
					dw_RequestValue = dw_RequestValue - i_ReturnValue;
				}
			}
			while ((dw_NbrOfReceiveValues < dw_NbrOfValuesToReceive) && (i_ReturnValue != SOCKET_ERROR));

            printf("\n");

			if (i_ReturnValue != SOCKET_ERROR)
			{

		        /* Test if the data format time stamp is activated */
			    if (ps_ThreadInfo->dw_DataFormat & 1)
				{
                    DisplayTimeStamp (s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[0],s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[1]);
				    printf("Counter: %lu", s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%f%s",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampRealData.f_RealValue [dw_Cpt],pc_Unit[(ps_ThreadInfo->dw_ChannelTypeMask >> dw_Cpt) & 1]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_BinaryValue[dw_Cpt]);
						}
					}
				}
				else
				{
				    printf("Counter: %lu", s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%f%s",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampRealData.f_RealValue [dw_Cpt],pc_Unit[(ps_ThreadInfo->dw_ChannelTypeMask >> dw_Cpt)&1]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_BinaryValue [dw_Cpt]);
						}
					}
				}
			}
			else
			{
				printf("Socket Error\n");
        		ps_ThreadInfo->b_Exit = 1;
            }

        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }


DWORD WINAPI dw_SOAPThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo * ps_ThreadInfo;
		INT i_ReturnValue             = 0;
    	DWORD dw_Cpt                  = 0;
		CHAR *pc_Unit[2] = {"V","mA"};
		DWORD  dw_UsedCpt;
		struct MSXE302x__AnalogInputGetAutoRefreshValuesResponse Response;
		float * pf_Value =  (float *) Response.ulValue;

        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {

            printf("\n");

			i_ReturnValue = soap_call_MSXE302x__AnalogInputGetAutoRefreshValues(ps_ThreadInfo->ps_SoapEnv, ps_ThreadInfo->pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response);

			if (!(i_CheckAndPrintError (ps_ThreadInfo->ps_SoapEnv, ps_ThreadInfo->pc_SOAPIPAddress, SOAP_ACTION, i_ReturnValue, Response.sResponse.iReturnValue, Response.sResponse.syserrno, "soap_call_MSXE302x__AnalogInputGetAutoRefreshValues", 1)))
			{
				dw_UsedCpt = 0;

		        /* Test if the data format time stamp is activated */
			    if (ps_ThreadInfo->dw_DataFormat & 1)
				{
                    DisplayTimeStamp (Response.ulTimeStampHigh,Response.ulTimeStampLow);
				}

				printf("Counter: %lu", Response.ulCounterValue);

				/* Test if receive analog value */
				if (ps_ThreadInfo->dw_DataFormat & 2)
				{
					for (dw_Cpt = 0; dw_Cpt < 16; dw_Cpt ++)
					{ 
						if ((ps_ThreadInfo->dw_ChannelMask >> dw_Cpt) & 0x1)
						{
							printf(" %lu:%f%s",dw_Cpt, pf_Value[dw_Cpt],pc_Unit[(ps_ThreadInfo->dw_ChannelTypeMask >> dw_UsedCpt) & 1]);
							dw_UsedCpt ++;
						}
					}
				}
				else
				{
					for (dw_Cpt = 0; dw_Cpt < 16; dw_Cpt ++)
					{ 
						if ((ps_ThreadInfo->dw_ChannelMask >> dw_Cpt) & 0x1)
						{
							printf(" %lu:%lX",dw_Cpt, Response.ulValue[dw_Cpt]);
						}
					}
				}
			}
			else
			{
				printf("Socket Error\n");
        		ps_ThreadInfo->b_Exit = 1;
            }

			/* 
			 * gSOAP doesn't release automatically the memory that 
			 * it allocates to deserialize data, that's why both 
			 * following functions have to be used to release the 
			 * allocated memory.
			 */
			soap_destroy(ps_ThreadInfo->ps_SoapEnv);
			soap_end(ps_ThreadInfo->ps_SoapEnv);
        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }



void main(void)
{
	INT i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	CHAR pc_IPAddressTemp[50];
	CHAR pc_IPAddress[50];
	CHAR pc_SOAPIPAddress[50];
	SOCKET  s_ClientSocket;
	DWORD dw_Dummy = 0;
	xsd__unsignedLong dw_ChannelMask = 0;
    struct MSXE302x__unsignedLong16FixedArrayParam ul16_GainArray;
    struct MSXE302x__unsignedLong16FixedArrayParam ul16_PolarityArray;
	xsd__unsignedLong dw_AverageMode = 0;
	xsd__unsignedLong dw_AverageValue = 0;
    xsd__unsignedLong dw_ConversionTime =0;
    xsd__unsignedLong dw_ConversionTimeUnit =0;
	xsd__unsignedLong dw_TriggerMask = 0; 
	xsd__unsignedLong dw_TriggerMode = 0; 
	xsd__unsignedLong dw_HardwareTriggerEdge = 0;
	xsd__unsignedLong dw_HardwareTriggerCount = 0;
	xsd__unsignedLong dw_ByTriggerNbrOfSeqToAcquire = 0;
	xsd__unsignedLong dw_DataFormat = 0;
	xsd__unsignedLong dw_OptionParameter1 = 0;
	xsd__unsignedLong dw_OptionParameter2 = 0;
	xsd__unsignedLong dw_OptionParameter3 = 0;
	DWORD  dw_AcquisitionMode;
	struct MSXE302x__Response Response;
	struct MSXE302x__AnalogInputGetChannelTypeResponse GetChannelTypeResponse;
	DWORD dw_NumberOfChannelsToAcquire = 0;
    DWORD dw_Cpt = 0;
    INT i_key = 0;
    DWORD    dw_ThreadId               = 0;
    HANDLE   h_ThreadHandle            = 0;
    str_ThreadInfo   s_ThreadInfo;
	CHAR * pc_ChannelType[2] = {"VOLTAGE","CURRENT"};
	DWORD dw_ChannelTypeMask = 0;


	/**************************/
	/* Get the console handle */
	/**************************/
	h_ConsoleHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	h_ConsoleMutex  = CreateMutex (NULL, FALSE, "MSX-E302x AutoRefresh"); 
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E302x AutoRefresh                             |\n");
	printf("| MSX-E302x Auto Refresh Test Program               |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
		if (!(i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 1)))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

			i_Error = soap_call_MSXE302x__AnalogInputGetChannelType(ps_SoapEnv,pc_SOAPIPAddress,SOAP_ACTION,0,&GetChannelTypeResponse);
			if (!(i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, GetChannelTypeResponse.sResponse.iReturnValue, GetChannelTypeResponse.sResponse.syserrno, "soap_call_MSXE302x__AnalogInputGetChannelType", 0)))
			{
				printf("Channels type : \n");
				for (dw_Cpt = 0; dw_Cpt < NBR_OF_CHANNELS; dw_Cpt ++)
				{
					printf("Channel %lu : %s\n",dw_Cpt,pc_ChannelType[GetChannelTypeResponse.ulType[dw_Cpt]]);
				}


			
				/* Get the auto refresh initialisation parameter */
				GetAutoRefreshParameters(GetChannelTypeResponse.ulType,
				                         &dw_ChannelMask, 
				                         &dw_ChannelTypeMask, 
										 &ul16_GainArray.ulValue[0],
										 &ul16_PolarityArray.ulValue[0],
										 &dw_AverageMode, 
										 &dw_AverageValue, 
										 &dw_ConversionTime,
										 &dw_ConversionTimeUnit,
										 &dw_TriggerMask, 
										 &dw_TriggerMode, 
										 &dw_HardwareTriggerEdge, 
										 &dw_HardwareTriggerCount, 
										 &dw_ByTriggerNbrOfSeqToAcquire,
										 &dw_DataFormat,
										 &dw_AcquisitionMode);

				/* Initialise the socket to receive the data */

				if (dw_AcquisitionMode == 1)
				{
					i_Error = i_Socket_ClientOpen(pc_IPAddress,8989,&s_ClientSocket);
				}

				if (i_Error == 0)
				{
				   /* Define the number of data to receive */
					dw_NumberOfChannelsToAcquire = 0;

					for (dw_Cpt = 0; dw_Cpt < NBR_OF_CHANNELS; dw_Cpt ++)
					{
						if (dw_ChannelMask & (1<<dw_Cpt))
							dw_NumberOfChannelsToAcquire ++;
					}

					s_ThreadInfo.s_ClientSocket = s_ClientSocket;
					s_ThreadInfo.dw_NumberOfChannelsToAcquire = dw_NumberOfChannelsToAcquire;
					s_ThreadInfo.dw_DataFormat                = dw_DataFormat;
					s_ThreadInfo.dw_ChannelTypeMask           = dw_ChannelTypeMask;
					s_ThreadInfo.b_Exit                       = 0;
					s_ThreadInfo.ps_SoapEnv                   = ps_SoapEnv;
					s_ThreadInfo.pc_SOAPIPAddress             = pc_SOAPIPAddress;
					s_ThreadInfo.dw_ChannelMask               = dw_ChannelMask;


					printf("\n Press ESC to exit the sample...");

					if (dw_AcquisitionMode == 1)
					{
						/*************************
						 * Create the data thread
						 *************************/
						h_ThreadHandle = CreateThread (NULL,
					 		        				   0,
									   				   dw_DataClientThreadProc,
													   &s_ThreadInfo,
													   0,
													   &dw_ThreadId);
       
						Sleep (100);
					}

					/* Start the auto refresh */
					i_Error = soap_call_MSXE302x__AnalogInputInitAndStartAutoRefresh (ps_SoapEnv,
																					  pc_SOAPIPAddress, 
																					  SOAP_ACTION,
																					  dw_ChannelMask, 
																					  &ul16_GainArray,
																					  &ul16_PolarityArray,
																					  dw_AverageMode,
																					  dw_AverageValue, 
																					  dw_ConversionTime, 
																					  dw_ConversionTimeUnit, 
																					  dw_TriggerMask, 
																					  dw_TriggerMode, 
																					  dw_HardwareTriggerEdge, 
																					  dw_HardwareTriggerCount, 
																					  dw_ByTriggerNbrOfSeqToAcquire,
																					  dw_DataFormat, 
																					  dw_OptionParameter1, 
																					  dw_OptionParameter2, 
																					  dw_OptionParameter3, 
																					  &Response);

					if (!(i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MSXE302x__AnalogInputInitAndStartAutoRefresh", 0)))
					{
						if (dw_AcquisitionMode == 0)
						{
							/*************************
							 * Create the data thread
							 *************************/
							h_ThreadHandle = CreateThread (NULL,
					 			        				   0,
										   				   dw_SOAPThreadProc,
														   &s_ThreadInfo,
														   0,
														   &dw_ThreadId);
						}

						do
						{
							if (kbhit()) 
							{
								i_key = getch();
							}
						}
						while ((i_key != 27) && (!s_ThreadInfo.b_Exit));

                    
						s_ThreadInfo.b_Exit = 1;


						Sleep (100);
					
						/* Free the thread */
						/* Test if thread installed */
						if (h_ThreadHandle != 0)
						{
							/* Close the thread handle */
							CloseHandle (h_ThreadHandle);
           
						}
                    
                    
						i_Error = soap_call_MSXE302x__AnalogInputStopAndReleaseAutoRefresh(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
						i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, Response.syserrno, "soap_call_MSXE302x__AnalogInputStopAndReleaseAutoRefresh", 0);

					}

					if (dw_AcquisitionMode == 1)
					{
						/* Close the socket connection */
						i_Socket_ClientClose (s_ClientSocket);
					}
				}
				else
				{
					printf("i_Socket_ClientOpen Error\n");
				}
			}
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/*Print End programme informations*/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

	/* Close the console handle */
	if (h_ConsoleHandle != NULL)
	{
		/* Close the console handle */
		CloseHandle(h_ConsoleHandle);
		CloseHandle(h_ConsoleMutex);
	}
}
