/** @file TCP_UDP_Client.c
* AutoRefresh : MX370x : TCP/UDP client
* 
*
* @author J. Krauth
* @date 14.01.2008
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MSXE302x.nsmap"
#include "..\util\socket.h"
#include "..\util\utility.h"



/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/



INT i_UTILS_GetTheNumberOfChannels (PCHAR pc_ModuleType, DWORD * pdw_NbrOfChannels)
{
	DWORD dw_Cpt = 0;
	CHAR pc_NbrOfChannels [5];
	DWORD dw_NbrOfChannelIndex = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_NbrOfChannelIndex = 0; dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ); dw_NbrOfChannelIndex ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex] == '-')
				break;
			else
				pc_NbrOfChannels [dw_NbrOfChannelIndex] = pc_ModuleType [dw_Cpt + dw_NbrOfChannelIndex];

		}
		pc_NbrOfChannels[dw_NbrOfChannelIndex] = '\0';

		if (dw_NbrOfChannelIndex < (strlen(pc_ModuleType) - dw_Cpt ))
			*pdw_NbrOfChannels = atol (pc_NbrOfChannels);
		else
			return -1;
	}
	else
		return -1;
	
	return 0;
}


typedef struct
      {
      DWORD   dw_DataFormat;
      DWORD   dw_NumberOfChannelsToAcquire;
      BYTE    b_Exit;
      SOCKET  s_ClientSocket;
	  BYTE    b_SaveValues;
	  BYTE    b_Mode;
	  struct  sockaddr_in *sai_Server;
	  DWORD	  dw_StructSize;
      }str_ThreadInfo;



#pragma pack(1)
typedef union
      {
      struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtTimeStampBinaryData;

	 struct
	    {
	    DWORD dw_TimeStamp[2];
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtTimeStampRealData;

	 struct
	    {
	    DWORD dw_Counter;
	    DWORD dw_BinaryValue[16];
	    }s_WichtoutTimeStampBinaryData;


	 struct
	    {
	    DWORD dw_Counter;
	    FLOAT  f_RealValue[16];
	    }s_WichtoutTimeStampRealData;
	 }str_AutoRefreshData;
#pragma pack()


DWORD WINAPI dw_ThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo * ps_ThreadInfo;
		INT i_ReturnValue             = 0;
    	DWORD dw_Cpt                  = 0;
    	DWORD dw_RequestValue         = 0;
    	DWORD dw_NbrOfReceiveValues   = 0;
    	DWORD dw_NbrOfValuesToReceive = 0;
		str_AutoRefreshData s_AutoRefreshData;
		FILE *fp;

        /* Variable initialisation */
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
		if (ps_ThreadInfo->b_SaveValues)
		{
			fp = fopen ("values.csv", "w+");
			if (fp == NULL)
			{
				printf ("fopen error\n");
				return (0);
			}
		}

        do
        {
		    /* Test if the data format time stamp is activated */
			if (ps_ThreadInfo->dw_DataFormat & 1)
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 3) * sizeof (DWORD);
			}
			else
			{
                dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 1) * sizeof (DWORD);
			}
				
			dw_RequestValue = dw_NbrOfValuesToReceive;
			dw_NbrOfReceiveValues = 0;

			do
			{
				
				if (ps_ThreadInfo->b_Mode == _UDP_MODE)		
						i_ReturnValue = recvfrom (ps_ThreadInfo->s_ClientSocket, (PCHAR)&s_AutoRefreshData.s_WichtoutTimeStampBinaryData, dw_RequestValue, 0, (struct sockaddr *) ps_ThreadInfo->sai_Server, &ps_ThreadInfo->dw_StructSize);
				
				if (ps_ThreadInfo->b_Mode == _TCP_MODE)		
						i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, (PCHAR)&s_AutoRefreshData.s_WichtoutTimeStampBinaryData, dw_RequestValue, 0); 

				if (i_ReturnValue != SOCKET_ERROR)
				{
					dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
					dw_RequestValue = dw_RequestValue - i_ReturnValue;
				}
			}
			while ((dw_NbrOfReceiveValues < dw_NbrOfValuesToReceive) && (i_ReturnValue != SOCKET_ERROR));

            printf("\n");

			if (i_ReturnValue != SOCKET_ERROR)
			{

		        /* Test if the data format time stamp is activated */
			    if (ps_ThreadInfo->dw_DataFormat & 1)
				{					
					if (ps_ThreadInfo->b_SaveValues)
					{
						fprintf (fp,"%s;", DisplayTimeStampCSV (s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[0],s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[1]));
					}
					else
						DisplayTimeStamp (s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[0],s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_TimeStamp[1]);

				    printf("Counter: %lu", s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_Counter);
					if (ps_ThreadInfo->b_SaveValues)
						fprintf(fp, "Counter: %lu;", s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%fV",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampRealData.f_RealValue [dw_Cpt]);
							if (ps_ThreadInfo->b_SaveValues)
								fprintf(fp, "%lu;%f;V or mA;",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampRealData.f_RealValue [dw_Cpt]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_BinaryValue[dw_Cpt]);
							if (ps_ThreadInfo->b_SaveValues)
								fprintf(fp, "%lu;%lX;",dw_Cpt,s_AutoRefreshData.s_WichtTimeStampBinaryData.dw_BinaryValue[dw_Cpt]);
						}
					}
				}
				else
				{
				    printf("Counter: %lu", s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_Counter);
					if (ps_ThreadInfo->b_SaveValues)
						fprintf(fp, "Counter: %lu;", s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_Counter);

					/* Test if receive analog value */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%fV",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampRealData.f_RealValue [dw_Cpt]);
							if (ps_ThreadInfo->b_SaveValues)
								fprintf(fp, "%lu;%f;V or mA;",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampRealData.f_RealValue [dw_Cpt]);
						}
					}
					else
					{
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							printf(" %lu:%lX",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_BinaryValue [dw_Cpt]);
							if (ps_ThreadInfo->b_SaveValues)
								fprintf(fp, "%lu;%lX;",dw_Cpt,s_AutoRefreshData.s_WichtoutTimeStampBinaryData.dw_BinaryValue [dw_Cpt]);
						}
					}
				}
				if (ps_ThreadInfo->b_SaveValues)
					fprintf(fp,"\n");
			}
			else
			{
				/* Abnormal error, print it. */
				if (ps_ThreadInfo->b_Exit != 1)
					printf("Socket Error\n");

        		ps_ThreadInfo->b_Exit = 1;
            }

        } 
        while (ps_ThreadInfo->b_Exit == 0);

		if (ps_ThreadInfo->b_SaveValues)
			fclose (fp);

        /* Return from thread */
        return (0);

        }



void main(void)
{
	CHAR              pc_IPAddressTemp [50];
	CHAR              pc_IPAddress     [50];
	CHAR              pc_SOAPIPAddress [50];
	INT                i_key                        = 0;
	INT                i_Error                      = 0;
	INT                i_ReturnValue                = 0;
	HANDLE             h_ThreadHandle               = NULL;
	SOCKET             s_ClientSocket;
	DWORD             dw_Cpt                        = 0;
	DWORD             dw_Dummy                      = 0;
	DWORD             dw_ThreadId                   = 0;
	DWORD             dw_NbrOfChannels              = 0;
	DWORD			  dw_Choice			            = 0;
	WORD			  w_Port			            = 0;
	BYTE			  b_Mode					    = _TCP_MODE;
	xsd__unsignedLong dw_DataFormat                 = 0;
	xsd__unsignedLong dw_ChannelMask                = 0;
	xsd__unsignedLong dw_AverageMode                = 0;
	xsd__unsignedLong dw_AverageValue               = 0;
	xsd__unsignedLong dw_DivisionFactor             = 0;
	xsd__unsignedLong dw_TriggerAction              = 0; 
	xsd__unsignedLong dw_ReservedParameter2         = 0;
	xsd__unsignedLong dw_ReservedParameter3         = 0;
	xsd__unsignedLong dw_ReservedParameter4         = 0;
	xsd__unsignedLong dw_HardwareTriggerCount       = 0;
	xsd__unsignedLong dw_TransducerTypeSelection    = 0;
	xsd__unsignedLong dw_HardwareTriggerFilterTime  = 0;
	xsd__unsignedLong dw_ByTriggerNbrOfSeqToAcquire = 0;
	struct soap      *ps_SoapEnv                    = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
    str_ThreadInfo   s_ThreadInfo;
	DWORD dw_StructSize = 0;
	struct sockaddr_in sai_Server;

	memset (&s_ThreadInfo, 0, sizeof (s_ThreadInfo));

	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSXE302x TCP/UDP Client                           |\n");
	printf("| This client permits to read data on a TCP or UDP  |\n");
	printf("| socket.                                           |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Port */
        do 
        {
            printf("\nGet the port number: ");
            scanf ("%d",&w_Port);
        }
        while (w_Port < 1);

		/* Connection mode */
        do 
        {
            printf("\nGet the connection mode (0=TCP, 1=UDP): ");
            scanf ("%d",&b_Mode);
        }
        while ((b_Mode < 1) && (b_Mode >= 2));

		if (b_Mode == 0)
		   b_Mode = _TCP_MODE;

		if (b_Mode == 1)
		   b_Mode = _UDP_MODE;

		/* Get the number of channels */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			

			printf("\n in Sequence mode, the sequence counter must be activated !");

				do 
				{
					printf("\nGet the number of channels acquired: ");
					scanf ("%d",&dw_Choice);
				}
				while ((dw_Choice < 1) && (dw_Choice > 16));
				s_ThreadInfo.dw_NumberOfChannelsToAcquire = dw_Choice;

				/* Options */
				do 
				{
					printf("\nDo You want to receive a time stamp (0: No, 1: Yes) ?");
					scanf ("%d",&dw_Choice);
				}
				while (dw_Choice > 1);

				dw_DataFormat = dw_Choice;

				do 
				{
					printf("\nDo You want to receive the value in digital form (tape 0) or in analog form V (type 1) ?");
					scanf ("%d",&dw_Choice);
				}
				while (dw_Choice > 1);

				dw_DataFormat = dw_DataFormat | (dw_Choice << 1);
				
				do 
				{
					printf("\nSave values in a file (0: No, 1: Yes) ?");
					scanf ("%d",&s_ThreadInfo.b_SaveValues);
				}
				while (s_ThreadInfo.b_SaveValues > 1);


				/* Initialise the socket to receive the data */
				if (b_Mode == _TCP_MODE)
					i_Error = i_Socket_ClientOpen(pc_IPAddress, w_Port, &s_ClientSocket);

				if (b_Mode == _UDP_MODE)
					i_Error = i_Socket_ClientOpenUDP(pc_IPAddress, w_Port, &s_ClientSocket, &sai_Server);

				if (i_Error == 0)
				{
					s_ThreadInfo.b_Exit         = 0;
					s_ThreadInfo.dw_DataFormat  = dw_DataFormat;
					s_ThreadInfo.s_ClientSocket = s_ClientSocket;
					s_ThreadInfo.b_Mode         = b_Mode;	
					s_ThreadInfo.dw_StructSize  = sizeof (struct sockaddr_in);
					s_ThreadInfo.sai_Server = &sai_Server;

					printf("\n Press ESC to exit the sample...");

					/* Create the data thread */
					h_ThreadHandle = CreateThread (NULL,
							 	        		   0,
										   	       dw_ThreadProc,
											       &s_ThreadInfo,
											       0,
												   &dw_ThreadId);
   
				

					if (h_ThreadHandle != NULL)
					{
		                do
				        {
							if (kbhit()) 
							{
								i_key = getch();
						    }
							else
							{
								Sleep (100);
						    }
		                }
				        while ((i_key != 27) && (!s_ThreadInfo.b_Exit));


						s_ThreadInfo.b_Exit = 1;


						Sleep (100);
			
				        /* Free the thread */
				        if (h_ThreadHandle != 0)
					    {
							/* Close the thread handle */
					        CloseHandle (h_ThreadHandle);
			
						}

					}
					else
					{
						printf("Fail to create the data read thread\n");
					}

					/* Close the socket connection */
					i_Socket_ClientClose (s_ClientSocket);
				}
				else
				{
					printf("i_Socket_ClientOpen Error\n");
				}

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, s_Response error :%d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

}
