

#ifndef __SAMPLE_UTILITY_H_
	#define __SAMPLE_UTILITY_H_

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Display a timeval struct in human string
 *
 * Convert and save date and time element
 * of a timeval struct in comprehensible outputs:
 * day/month/years hour:min.:sec.:msec.:usec.
 */
char *DisplayTimeStamp (DWORD TimeStampHigh, DWORD TimeStampLow)
{
	struct tm *tm_ptr;
	time_t useconds, mseconds;
    CHAR *pc_Write = malloc(300);
       
       tm_ptr = localtime (&TimeStampHigh);
       useconds = TimeStampLow;       
       mseconds = useconds/1000;
       useconds -= (mseconds * 1000);
       printf ("%02d/%02d/%04d-%02d:%02d:%02d:%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);
       sprintf(pc_Write,"%02d/%02d/%04d-%02d:%02d:%02d:%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);

       return pc_Write;


}

/** Display a timeval struct in human string, return a CSV string
 *
 * Convert and save date and time element
 * of a timeval struct in comprehensible outputs:
 * day/month/years hour:min.:sec.:msec.:usec.
 */
char *DisplayTimeStampCSV (DWORD TimeStampHigh, DWORD TimeStampLow)
{
	struct tm *tm_ptr;
	time_t useconds, mseconds;
    CHAR *pc_Write = malloc(300);
       
       tm_ptr = localtime (&TimeStampHigh);
       useconds = TimeStampLow;       
       mseconds = useconds/1000;
       useconds -= (mseconds * 1000);
       printf ("%02d/%02d/%04d-%02d:%02d:%02d:%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);
       sprintf(pc_Write,"%02d/%02d/%04d;%02d:%02d:%02d;%03ld,%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);

       return pc_Write;


}


/** Convert a timeval struct in human string
 *
 * Convert and save date and time element
 * of a timeval struct in comprehensible outputs:
 * day/month/years hour:min.:sec.:msec.:usec.
 */
char *ConvertTimeStamp (DWORD TimeStampHigh, DWORD TimeStampLow)
{
	struct tm *tm_ptr;
	time_t useconds, mseconds;
    static CHAR c_Write [300];
       
       tm_ptr = localtime (&TimeStampHigh);
       useconds = TimeStampLow;       
       mseconds = useconds/1000;
       useconds -= (mseconds * 1000);
       sprintf(c_Write,"%02d/%02d/%04d:%02d:%02d:%02d,%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);

       return c_Write;


}


//-------------------------------------------------------------------------------------------------------
//										Utility functions
//-------------------------------------------------------------------------------------------------------

/** Format the IP address.
 *
 * @param [in]	pc_IPAddress	: The IP address in string form.
 * @param [out]	pc_NewIPAddress	: The formatted IP address in string form.
 */
static VOID v_UTILS_ClearNullInIPAddress(PCHAR pc_IPAddress, PCHAR pc_NewIPAddress)
{
	unsigned char b_Cpt = 0;
	unsigned char b_Index = 0;
	unsigned char b_ClearNullFlag = 1;
	unsigned char b_IPIsNull = 1;

	while (pc_IPAddress[b_Cpt] != '\0')
	{
		if (pc_IPAddress[b_Cpt] == '.')
		{
			if (b_IPIsNull)
				pc_NewIPAddress[b_Index++] = '0';

			pc_NewIPAddress[b_Index++] = pc_IPAddress[b_Cpt];
			b_ClearNullFlag = 1;
			b_IPIsNull = 1;
		}
		else
	    {
			if ((pc_IPAddress[b_Cpt] == '0') && (b_ClearNullFlag == 0))
			{
				pc_NewIPAddress[b_Index++] = pc_IPAddress[b_Cpt];
			}
		
			if (pc_IPAddress[b_Cpt] != '0') 
			{
				pc_NewIPAddress[b_Index++] = pc_IPAddress[b_Cpt];
				b_ClearNullFlag = 0;
				b_IPIsNull = 0;
			}
		}
		b_Cpt ++;
	}
	pc_NewIPAddress[b_Index] = '\0';
}

//-------------------------------------------------------------------------------------------------------
//                                       SOAP function                                                             
//-------------------------------------------------------------------------------------------------------

#define SOAP_ACTION ""

static int i_SOAP_Init(struct soap **ps_SoapEnv)
{
	/* Initialise the SOAP connection */
	*ps_SoapEnv = soap_new();

	if (*ps_SoapEnv != NULL)
	{
		soap_init2(*ps_SoapEnv, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
		(*ps_SoapEnv) -> send_timeout = 1;
		(*ps_SoapEnv) -> recv_timeout = 1;
		(*ps_SoapEnv) -> accept_timeout = 1;
		return 0;
	}
	printf ("Fail to initialize the SOAP connection\n");
	return -1;
}

//-------------------------------------------------------------------------------------------------------

static int i_SOAP_Close(struct soap *ps_SoapEnv)
{
	soap_destroy(ps_SoapEnv);
	soap_end(ps_SoapEnv);
	soap_free(ps_SoapEnv);
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Test error numbers.
 *
 * Test error numbers of a soap function (soap_call_...) and display the appropriated error message. 
 *
 * @param [in]	soap					: A pointer on the soap structure.
 * @param [in]	soap_endpoint			: Module IP address.
 * @param [in]	soap_endpoint			: Soap action.
 * @param [in]	soapCallError			: Return value from a soap_call_... function.
 * @param [in]	responseReturnValue		: Return value (generally iReturnValue) from the response structure from a soap_call_... function.
 * @param [in]	syserrno				: System errno value (generally syserrno) from the response structure from a soap_call_... function.
 * @param [in]	functionName			: Name of the tested function, this will be displayed in the error message.
 * @param [in]	warnNoError				: 0: Display a message when no error (e.g: printf("\n %s OK ", functionName);).\n 
 *										  1: No message when no error.
 *
 * @retval 0: No error.
 * @retval 1: Error.
 */
int i_CheckAndPrintError (struct soap *soap, const char *soap_endpoint, const char *soap_action, int soapCallError, int responseReturnValue, int syserrno, const char *functionName, char warnNoError)
{
	struct MXCommon__ByteArrayResponse         s_ByteArrayResponse;

	if ((soapCallError == 0) && (responseReturnValue == 0))
	{
		if (!warnNoError)
			printf("\n %s OK ", functionName);
	}
	else
	{
		printf("\n %s SOAP error: %d,Response error: %d\n", functionName, soapCallError,responseReturnValue);
		if (soapCallError != 0)
			printf("\nSOAP Error : %s", *soap_faultstring(soap));
		else
		{
				if ((responseReturnValue == -1) || (responseReturnValue <= -100) && (syserrno != 0))
				{
					soapCallError = soap_call_MXCommon__Strerror(soap, soap_endpoint, soap_action, syserrno, &s_ByteArrayResponse);
					if (soapCallError != 0)
						printf("\nSOAP Error : %s", *soap_faultstring(soap));
					else
						printf("\nError : %s",s_ByteArrayResponse.sArray.__ptr);
				}
		}

		return 1;
	}

	return 0;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

#endif // #ifndef __SAMPLE_UTILITY_H_